# Generated by Django 3.2.13 on 2022-07-25 12:15

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('messaging', '0002_add_topic_and_message'),
    ]

    operations = [
        migrations.AlterField(
            model_name='message',
            name='content_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='sent_messages', to='contenttypes.contenttype'),
        ),
        migrations.AlterField(
            model_name='message',
            name='object_id',
            field=models.CharField(max_length=128, null=True),
        ),
    ]
